/*****************************************************************************
**               AMICCOM Electronics Corporation Document                   **
**          Copyright (c) 2011-2016 AMICCOM Electronics Corporation         **
**                                                                          **
**      A3,1F,No.1, Li-Hsin 1th Road, Science_Based Industrid Park,         **
**                       Hsin_Chu City, 300, Taiwan, ROC.                   **
**                 Tel: 886-3-5785818   Fax: 886-3-5785819                  **
**         E-mail:info@amiccom.com.tw  http: //www.amiccom.com.tw           **
*****************************************************************************/

#include <intrins.h>
#include <absacc.h>
#include "..\include\include.h"

//-----------------------------------------------------------------------------
// Flash_Protect_Battery:
//      return the Battery Detect Value (threshold = 2.2V)
//  return:
//          1: REGI >= 2.2V
//          0: REGI  < 2.2V
//  BATTERY_REG:
//      
//-----------------------------------------------------------------------------
Uint8 Flash_Protect_Battery(void)
{
    Uint8 tmp;

    XBYTE[STROBE_REG] = CMD_STBY;
    XBYTE[BATTERY_REG] = 0x05; 
    do{
        tmp = XBYTE[BATTERY_REG] & 0x01;
    }while(tmp);
    tmp = (XBYTE[BATTERY_REG] & 0x20) >> 5;
    Delay15us(4);
    return tmp;
}

//-----------------------------------------------------------------------------
// MUST run FlashTiming 1 times before any flash control
//-----------------------------------------------------------------------------
void FlashTiming(Uint8 mcu_speed, Uint8 program_time, Uint8 erase_time )
{
    EA = 0;
    Write9B(mcu_speed);     // set system clock frequency
    Write9C(program_time);  // set program time
    Write9D(erase_time);    // set erase time
    EA = 1;
}


//-----------------------------------------------------------------------------
// ProgramBuffer:
//      Write buf[] to CODE[addr]
//  addr:   the address of flash. addr must be even.
//  buf[]:  the data buffer
//  length: the length of data buffer. length must be even.
//-----------------------------------------------------------------------------
Uint8 ProgramBuffer(Uint16 addr, Uint8 buf[], Uint8 length)
{
    idata Uint8 xdata *x_ptr;
    idata Uint8* buf_ptr;
    idata Uint8 size;

    buf_ptr = buf;    
    size = length;

    // program 2 bytes
    EA = 0;
    x_ptr = (Uint8*)addr;           // set point
    PCON |= BIT4;                   // PWE=1
    Write9A(WRITE_BYTE);
    while(size --)
    {
        *x_ptr++ = *buf_ptr++;
    }
    PCON &= ~BIT4;                  // PWE=0
    EA = 1;

    return PASS;
}

//-----------------------------------------------------------------------------
// EraseSector:
//      Erase a sector (128 bytes)
//  addr:   the address of flash. 
//-----------------------------------------------------------------------------
Uint8 EraseSector(Uint16 addr)
{
    Uint8 xdata *ptr;

    EA = 0;
    ptr = (Uint8*)addr;
    PCON |= BIT4;                   // PWE=1
    Write9A(SECTOR_ERASE_WITH_PRE);
    *ptr = 0;
    PCON &= ~BIT4;                  // PWE=0
    EA = 1;
    
    return PASS;
}

//-----------------------------------------------------------------------------
// SectorErase:
//      Erase a sector (128 bytes) with flash protect
//  addr:   the address of flash. 
//-----------------------------------------------------------------------------
Uint8 SectorErase(Uint16 address)
{
    Uint8 i;

    // reset FLASHCTRL for safe
    _nop_();
    _nop_();
    TA = 0xAA;
    TA = 0x55;
    FLASHCTRL = FNOP;
    _nop_();
    _nop_();
    
    if(RSFLAG != 0)
        return FAIL;
    
    for(i = 0; i < 5; i++)
    {
        if(0 == Flash_Protect_Battery())
            return FAIL;
    }
    
    if(Check_stable != 0x5AA5)
        return FAIL;

    if(PASS != EraseSector(address))
        return FAIL;

    // reset FLASHCTRL for safe
    _nop_();
    _nop_();
    TA = 0xAA;
    TA = 0x55;
    FLASHCTRL = FNOP;  
    _nop_();
    _nop_();

    return PASS;
}

//-----------------------------------------------------------------------------
// WriteFlash:
//      Write buf[] to CODE[addr] with flash protect.
//  addr:   the address of flash. addr must be even.
//  buf[]:  the data buffer
//  length: the length of data buffer. length must be even.
//-----------------------------------------------------------------------------
Uint8 WriteFlash(Uint16 address, Uint8 buf[], Uint8 length)
{
    Uint8 i;
    
    if(0 != (address & 0x0001)) // Initial address must be even number
        return FAIL;

    if(length & 0x1)
        return FAIL;

    _nop_();
    _nop_();
    TA = 0xAA;
    TA = 0x55;
    FLASHCTRL = FNOP;
    _nop_();
    _nop_();
    
    if(RSFLAG != 0)
        return FAIL;
    
    for(i = 0; i < 5; i++)
    {
        if(0 == Flash_Protect_Battery())
            return FAIL;
    }
    
    if(Check_stable != 0x5AA5)
        return FAIL;

    if(FAIL == ProgramBuffer(address, buf, length))
        return FAIL;

    // reset FLASHCTRL for safe    
    _nop_();
    _nop_();
    TA = 0xAA;
    TA = 0x55;
    FLASHCTRL = FNOP;
    _nop_();
    _nop_();

    return PASS;
}

